using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Frequencies {
    public List<List<float>> tunerFrequencies = new List<List<float>>();

    public class Tone {
        public float frequency;
        public int group;
        public int index;

        public static bool operator ==(Tone t1, Tone t2) {
            return (t1.group == t2.group) && (t1.index == t2.index);
            // return t1.frequency == t2.frequency;
        }

        public static bool operator !=(Tone t1, Tone t2) {
            return (t1.group != t2.group) || (t1.index != t2.index);
            // return t1.frequency != t2.frequency;
        }
    }
    
    public Tone GetTone(int group, int index) {
        float frequency = tunerFrequencies[group][index];
        Tone tone = new Tone();
        tone.frequency = frequency;
        tone.group = group;
        tone.index = index;

        return tone;
    }

    public Tone[] GetRandomTones() {
        List<List<float>> freq = new List<List<float>>(tunerFrequencies);
        Tone[] random_frequencies = new Tone[tunerFrequencies.Count];
        List<int> used = new List<int>();
        while(used.Count < tunerFrequencies.Count) {
            int group = (int)(Random.value * freq.Count);
            if (used.Contains(group))
                continue;

            List<float> selected_group = freq[group];

            int index = (int)(Random.value * selected_group.Count);
            float frequency = selected_group[index];

            Tone tone = new Tone();
            tone.frequency = frequency;
            tone.group = group;
            tone.index = index;
            random_frequencies[used.Count] = tone;

            used.Add(group);
        }

        return random_frequencies;
    }

    public Frequencies() {
        List<float> low = new List<float>() {
            116.54f, //A#
            123.47f, //B
            130.81f, //C
            138.59f, //C#
            146.83f  //D
        };
        tunerFrequencies.Add(low);

        List<float> mid1 = new List<float>() {
            155.56f, //D#
            164.81f, //E
            174.61f, //F
            185.00f, //F#
            196.00f //G
        };
        tunerFrequencies.Add(mid1);

        List<float> mid2 = new List<float>() {
            207.65f, //G#
            220.00f, //A
            233.08f, //A#
            246.94f, //B
            261.63f  //C
        };
        tunerFrequencies.Add(mid2);

        List<float> high = new List<float>() {
            277.18f, //C#
            293.66f, //D
            311.13f, //D#
            329.63f, //E
            349.23f, //F
        };
        tunerFrequencies.Add(high);
    }

}
