using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using UnityEngine.SceneManagement;

public class GameController : MonoBehaviour {
    public SineGenerator mSineGenerator;
    public TunerController mTuner;
    public LockController mLock;

    public GameObject Instructions;
    public Transform InstructionsOnDesk;
    public Transform InstructionsOnScreen;

    public Transform CameraEndPosition;

    public GameObject PressSpaceToStart;

    public AudioClip WinningTune;
    AudioSource Audio;

    Frequencies mFrequencies;
    Puzzle mCurrentPuzzle;
	public Canvas retryCanvas;
	public Canvas solvedCanvas;

	public GameObject vault;
	private AlertController m_AlertController = null;
	private DoorController m_DoorController = null;
	private Timer m_Timer = null;
    public CameraController m_CameraController;

	void Awake(){
		retryCanvas.enabled = false;
	}

    public enum GameState {
        INSTRUCTIONS,
        TEST,
        INIT,
        SOLVE,
        FAILED,
        SOLVED,
		RESTART,
		WINNER
    }

    GameState mState;
    public GameState State {
        get {
            return mState;
        }

        set {
           switch (value) {
               case GameState.INSTRUCTIONS:
                   mTuner.enabled = false;
                   ShowInstructions();
                   break;
                case GameState.TEST:
                   HideInstructions();
                   mTuner.enabled = true;
                   PressSpaceToStart.active = true;
                   break;
               case GameState.INIT:
                   mTuner.enabled = false;
                    PressSpaceToStart.active = false;
                    // Play puzzle tune
                    mCurrentPuzzle = new Puzzle(); 
                    mCurrentPuzzle.Tones = mFrequencies.GetRandomTones();
                    mLock.SetPuzzle(mCurrentPuzzle);

                    StartCoroutine(PlayPuzzleTones(mCurrentPuzzle));
                    StartCoroutine(ChangeState(GameState.SOLVE, 5f));
                   break;
			case GameState.SOLVE:
				   m_Timer.StartTimer ();
                   mTuner.enabled = true;
            break;
			case GameState.FAILED:
				m_AlertController.StartAlert ();
				print ("GameState Is failed");
				StartCoroutine(ChangeState(GameState.RESTART, 5f));
                break;
			case GameState.SOLVED:
				m_CameraController.enabled = false;
				mTuner.enabled = false;
				mSineGenerator.enabled = false;
                  Cursor.visible = true;
                  iTween.MoveTo(Camera.main.gameObject, iTween.Hash("position", CameraEndPosition, "time", 3f));
                  iTween.RotateTo(Camera.main.gameObject, iTween.Hash("rotation", CameraEndPosition, "time", 3f));
                  Audio.PlayOneShot(WinningTune);
				StartCoroutine(ChangeState(GameState.WINNER, 10f));
                   break;
			case GameState.WINNER:
				print ("You winner are");
				solvedCanvas.enabled = true;
				break;

			case GameState.RESTART:
				print ("Show the dialog");
				m_CameraController.enabled = false;
                Cursor.visible = true;
				mTuner.enabled = false;
				mSineGenerator.enabled = false;
				retryCanvas.enabled = true;	
				break;
			  
           }
           mState = value;
        }
    }

    public class Puzzle {
        public Frequencies.Tone[] Tones;
        public int Time = 60;
    }

	// Use this for initialization
	void Start () {
        mFrequencies = new Frequencies();
	    State = GameState.INSTRUCTIONS;

		m_DoorController = (DoorController) vault.GetComponent(typeof(DoorController));
		m_AlertController = (AlertController) vault.GetComponent(typeof(AlertController));
		m_Timer = (Timer)GetComponent (typeof(Timer));

        Audio = GetComponent<AudioSource>();
	}
	
	// Update is called once per frame
	void Update () {
        switch(State) {
            case GameState.INSTRUCTIONS:
                if (Input.anyKey) {
                    State = GameState.TEST;
                }
                break;
            case GameState.TEST:
                if (Input.GetKeyDown(KeyCode.Space)) {
                    State = GameState.INIT;
                }
                break;
        }

		// TestVault();
	}

    IEnumerator PlayPuzzleTones(Puzzle puzzle) {
        foreach (Frequencies.Tone t in puzzle.Tones) {
            mSineGenerator.PlayFrequency(t.frequency, 1f);
            yield return new WaitForSeconds(1f);
        }
    }

    IEnumerator ChangeState(GameState state, float delay) {
        yield return new WaitForSeconds(delay);
        State = state;
    }

	void TestVault () {
		if (Input.GetKeyDown("space")){
			print("space key was pressed");
			UnlockVault ();

		}
	}

	public void CountDownFinished(){
		print ("Its a final countdown!");
		State = GameState.FAILED;
	}


	public void UnlockVault(){
		print ("Unlocking vault");
		if(m_DoorController.IsClosed()){
			m_DoorController.OpenSafe();
		}
		m_Timer.StopTimer ();
		StartCoroutine(ChangeState(GameState.SOLVED, 5f));

	}

    public void ShowInstructions() {
        // Instructions.transform.position = InstructionsOnScreen.position;
        // Instructions.transform.rotation = InstructionsOnScreen.rotation;
        iTween.MoveTo(Instructions, iTween.Hash("position", InstructionsOnScreen, "time", 0.5f));
        iTween.RotateTo(Instructions, iTween.Hash("rotation", InstructionsOnScreen, "time", 0.5f));
    }

    public void HideInstructions() {
        // Instructions.transform.position = InstructionsOnDesk.position;
        // Instructions.transform.rotation = InstructionsOnDesk.rotation;

        // iTween.MoveTo(Instructions, iTween.Hash("position", InstructionsOnDesk, "time", 0.5f));
        iTween.MoveTo(Instructions, iTween.Hash("position", InstructionsOnDesk, "time", 0.5f));
        iTween.RotateTo(Instructions, iTween.Hash("rotation", InstructionsOnDesk, "time", 0.5f));
    }

}
