﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class LockController : MonoBehaviour {
    public List<GameObject> Indicators;
    public List<Color> IndicatorColors;
    public GameController Game;

    AudioSource Audio;
    public AudioClip WrongCodeAudio;

    bool[] IndicatorStates;

    GameController.Puzzle puzzle = null;

    public void SetPuzzle(GameController.Puzzle p) {
        puzzle = p;
        ResetIndicators();
    }

    public void ResetIndicators() {
        IndicatorStates = new bool[Indicators.Count];
        for (int i = 0; i < Indicators.Count; i++)
            IndicatorStates[i] = false;
    }

    public void TryTone(Frequencies.Tone tone) {
        if (puzzle == null)
            return;

        bool incomplete = false;
        int found = -1;
        for (int i = 0; i < puzzle.Tones.Length; i++) {
            Frequencies.Tone puzzle_tone = puzzle.Tones[i];
            if (puzzle_tone == tone) {
                found = i;
                break;
            } else if (!IndicatorStates[i]) {
                incomplete = true;
            }
        }

        if (incomplete) {
            ResetIndicators();
            // if (found > 0)
            //     Audio.PlayOneShot(WrongCodeAudio);
        }

        if (found != -1) {
            IndicatorStates[found] = true; 
            
            bool disabled = false;
            // Disable indicators after the tested tone
            for (int i = found+1; i < Indicators.Count; i++) {
                if (IndicatorStates[i])
                    disabled = true;
                IndicatorStates[i] = false;
            }

            // if (disabled) Audio.PlayOneShot(WrongCodeAudio);
        }

    }

	// Use this for initialization
	void Start () {
        ResetIndicators();		
        Audio = GetComponent<AudioSource>();
	}
	
	// Update is called once per frame
	void Update () {
        int enabledCount = 0;
        for (int i = 0; i < Indicators.Count; i++) {
            MeshRenderer renderer = Indicators[i].GetComponent<MeshRenderer>();
            renderer.material.SetColor("_EmissionColor", IndicatorStates[i] ? IndicatorColors[i] : Color.grey);
            if (IndicatorStates[i]) enabledCount++;
        }

        if (enabledCount == Indicators.Count)
            Game.UnlockVault();
	}
}
