﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Oscilloscope : MonoBehaviour {
    public Texture2D mTexture;

    public Color Background;

    [Range(0,1)]
    public float Scaling;

    [Range(0,1)]
    public float HeightScaling;


    [Range(0,1000)]
    public float UpdateFreq;
    
    // Array used to clear the texture.
    Color[] clear;

    // Used for checking if its time to update
    float time = 0f;

    MeshRenderer renderer;

	void Start () {
        clear = new Color[mTexture.width*mTexture.height];
        for (int i = 0; i < mTexture.width*mTexture.height; i++) 
            clear[i] = Background;

        renderer = GetComponent<MeshRenderer>();
	}

    public void UpdateScope(float[] samples) {
        // Check if it's time for a update
        if (time < (UpdateFreq/1000f)) {
            time += Time.deltaTime;
            return;
        }

        // Reset time counter
        time = 0f;

        // Clear texture.
        mTexture.SetPixels(clear, 0);

        // If samples array is null draw a flat line.
        if (samples == null) {
            for (int x = 0; x < mTexture.width; x++) 
                mTexture.SetPixel(x, mTexture.height/2, Color.green);
            return;
        }


        int count = (int)(samples.Length*Scaling);
        int previous_x = 0;
        int previous_y = mTexture.height / 2;
        float wave_height = mTexture.height*HeightScaling;
        float y_offset = (mTexture.height-wave_height) / 2;
        for (int i = 0; i < count; i++) {
            int x = (int)(((float)mTexture.width / (float)count) * i);
            int y = (int)((samples[i]+1f/2f)*(mTexture.height*HeightScaling)+y_offset);
            mTexture.SetPixel(x, y, Color.green);
            
            // Linear interpolation se we get a line instead of dots
            int diff_x = x - previous_x;
            int diff_y = y - previous_y;
            int interop_length = (int) Mathf.Sqrt(Mathf.Pow(diff_x, 2) + Mathf.Pow(diff_y, 2));

            for (int a = 0; a < interop_length; a++) {
                float pos = (float) a / (float) interop_length;
                float x_pos = pos * diff_x;
                float y_pos = pos * diff_y;
                mTexture.SetPixel((int)(previous_x+x_pos), (int)(previous_y+y_pos), Color.green);
            }
            previous_x = x;
            previous_y = y;
        }

        mTexture.Apply();
    }
	
	// Update is called once per frame
	void Update () {
	}
}
