﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class SineGenerator : MonoBehaviour {
    public int SampleRate = 44100;
    public float a_frequency = 440.00f;

    public Oscilloscope mScope;

    [Range(0,1)]
    public float Gain = 0.05f;

    float frequency = 0.00f;
    float phase = 0f;
    public float[] samples;
    
    AudioSource audio;

    void Start() {
        AudioClip clip = new AudioClip();
        audio = GetComponent<AudioSource>();
        audio.clip = clip;
        audio.Play();
    }

    void Update() {
        mScope.UpdateScope(samples);
    }

    public void PlayFrequency(float freq, float duration) {
        frequency = freq;
        StartCoroutine(ChangeFrequency(0f, duration));
    }

    IEnumerator ChangeFrequency(float freq, float delay) {
        yield return new WaitForSeconds(delay);
        frequency = freq;
    }

    public void SetFrequency(float freq) {
        frequency = freq;
    }
    
    void OnAudioFilterRead(float[] data, int channels) {
        if (frequency == 0f) {
            for (int i = 0; i < data.Length; i++) {
                data[i] = 0f;
            }
            return;
        }

        float increment = frequency * 2f * Mathf.PI / SampleRate;
        for (int i = 0; i < data.Length; i=i+channels) {
            float value = Gain*Mathf.Sin(phase);
            data[i] = value;
            phase += increment;
            if (channels == 2) data[i+1] = value;
            if (phase > 2*Mathf.PI) phase = 0f;
        }

        samples = data;
    }

}
