﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class TunerController : MonoBehaviour {
    public GameObject[] Buttons;
    public GameObject[] Sliders;
    public SineGenerator mSineGenerator;

    AudioSource Audio;
    public AudioClip ButtonSelect;
    public AudioClip SwitchDown;
    public AudioClip SwitchUp;

    public Transform ButtonActivatedHeight;
    public Transform ButtonNormalHeight;
    public Transform[] SliderSteps;

    public LockController Lock;

    int[] SliderStates;
    public int ActiveButton;

    List<KeyCode> Keys = new List<KeyCode>() {
        KeyCode.Alpha1,
        KeyCode.Alpha2,
        KeyCode.Alpha3,
        KeyCode.Alpha4
    };

    Frequencies mFrequencies;

	// Use this for initialization
	void Start () {
        SliderStates = new int[4] {
            0, 0, 0, 0
        };

        mFrequencies = new Frequencies();
        Audio = GetComponent<AudioSource>();
	}

    public Frequencies.Tone GetCurrentTone(int button) {
        return mFrequencies.GetTone(button, SliderStates[button]);
    }
	
	// Update is called once per frame
	void Update () {
        for (int i = 0; i < Keys.Count; i++) {
            if (Input.GetKeyDown(Keys[i])) {
                ActiveButton = i;
                Audio.PlayOneShot(ButtonSelect);
            }
        }


        float wheel = Input.GetAxis("Mouse ScrollWheel");
        int slider_state = SliderStates[ActiveButton];
        if (wheel > 0) { 
            slider_state++;
            if (slider_state > 4)
                slider_state = 4;
            else
                Audio.PlayOneShot(SwitchUp);
        } else if (wheel < 0) {
            slider_state--;
            if (slider_state < 0)
                slider_state = 0;
            else
                Audio.PlayOneShot(SwitchDown);
        }
        SliderStates[ActiveButton] = slider_state;

        for (int i = 0; i < Buttons.Length; i++) {
            Vector3 position = Buttons[i].transform.localPosition;
            if (i == ActiveButton)
                position.y = ButtonActivatedHeight.localPosition.y;
            else
                position.y = ButtonNormalHeight.localPosition.y;
            Buttons[i].transform.localPosition = position;
        }

        for (int i = 0; i < Sliders.Length; i++) {
            Transform step = SliderSteps[SliderStates[i]];
            Vector3 pos = Sliders[i].transform.localPosition;
            pos.z = step.localPosition.z;
            Sliders[i].transform.localPosition = pos;
        }

        if (Input.GetMouseButton(0) || Input.GetMouseButton(1)) {
            Frequencies.Tone tone = GetCurrentTone(ActiveButton);
            mSineGenerator.SetFrequency(tone.frequency);
            Lock.TryTone(tone);
        } else {
            mSineGenerator.SetFrequency(0f);
        }
	}
}
